@@ -154,7 +154,7 @@ class DXJob(DXObject):
                 if isinstance(instance_type, basestring):
                     req_input["systemRequirements"] = {fn_name: {"instanceType": instance_type}}
                 elif isinstance(instance_type, dict):
-                    req_input["systemRequirements"] = {stage: {"instanceType": stage_inst} for stage, stage_inst in instance_type.iteritems()}
+                    req_input["systemRequirements"] = {stage: {"instanceType": stage_inst} for stage, stage_inst in instance_type.items()}
                 else:
                     raise DXError('Expected instance_type field to be either a string or a dict')
             if depends_on is not None:
@@ -258,7 +258,7 @@ class DXJob(DXObject):
 
         dxpy.api.job_set_properties(self._dxid, {"properties": properties}, **kwargs)
 
-    def wait_on_done(self, interval=2, timeout=sys.maxint, **kwargs):
+    def wait_on_done(self, interval=2, timeout=3600*24*7, **kwargs):
         '''
         :param interval: Number of seconds between queries to the job's state
         :type interval: integer
