@@ -15,143 +15,23 @@
 #   under the License.
 
 """
-DXWorkflow Handlers
-+++++++++++++++++++
+DXWorkflow Handler
+++++++++++++++++++
 
-Workflows contain metadata for a set of jobs to be run together.
-There are currently two types of workflows with two respective handler
-classes that are supported:
-
-* :class:`DXRecordWorkflow` for records created via the website of
-  type "pipeline" which contain metadata to run a series of jobs.
-  They can be run by calling the :func:`DXRecordWorkflow.run` method.
-  Inputs that have not been bound yet in the workflow need to be
-  provided using input field names of the form "N.name", where "name"
-  is the name of an input to the Nth stage (starting from 0).  Inputs
-  already bound in the workflow can be overridden by setting them here
-  as well.
-
-* :class:`DXAnalysisWorkflow` for new workflows that have class
-  "workflow".  These workflows create an analysis object when run and
-  package the resulting jobs together.
-
-The function :func:`DXWorkflow` can be used to return the appropriate
-class automatically.
+Workflows are data objects which contain metadata for a set of jobs to
+be run together.  They can be run by calling the
+:func:`DXWorkflow.run` method.
 
 """
 
-import json
-import copy
-from collections import OrderedDict
-from argparse import Namespace
 import re
-
 import dxpy
-from dxpy.bindings import DXDataObject, DXExecutable, DXAnalysis, get_handler
-from dxpy.exceptions import DXError, DXAPIError
-from dxpy.cli.exec_io import ExecutableInputs, stage_to_job_refs
+from dxpy.bindings import DXDataObject, DXExecutable, DXAnalysis
+from dxpy.exceptions import DXError
 
-####################
-# DXRecordWorkflow #
-####################
-
-class DXRecordWorkflow(DXDataObject, DXExecutable):
-    '''
-    Remote workflow object handler.  This class is used for the record
-    data objects with type "pipeline".
-    '''
-
-    _class = "record"
-
-    _describe = staticmethod(dxpy.api.record_describe)
-    _add_types = staticmethod(dxpy.api.record_add_types)
-    _remove_types = staticmethod(dxpy.api.record_remove_types)
-    _get_details = staticmethod(dxpy.api.record_get_details)
-    _set_details = staticmethod(dxpy.api.record_set_details)
-    _set_visibility = staticmethod(dxpy.api.record_set_visibility)
-    _rename = staticmethod(dxpy.api.record_rename)
-    _set_properties = staticmethod(dxpy.api.record_set_properties)
-    _add_tags = staticmethod(dxpy.api.record_add_tags)
-    _remove_tags = staticmethod(dxpy.api.record_remove_tags)
-    _close = staticmethod(dxpy.api.record_close)
-    _list_projects = staticmethod(dxpy.api.record_list_projects)
-
-    def run(self, workflow_input, project=None, folder="/", name=None, **kwargs):
-        '''
-        :param workflow_input: Hash of the workflow's input arguments, with keys equal to "N.name" where N is the stage number and name is the name of the input, e.g. "0.reads" if the first stage takes in an input called "reads"
-        :type workflow_input: dict
-        :param project: Project ID in which to run the jobs (project context)
-        :type project: string
-        :param folder: Folder in which the workflow's outputs will be placed in *project*
-        :type folder: string
-        :param name: String to append to the default job name for each job (default is the workflow's name)
-        :type name: string
-        :returns: list of job IDs in order of the stages
-
-        Run each stage in the associated workflow
-        '''
-
-        workflow_name = self.describe()['name']
-        workflow_spec = self.get_details()
-        workflow_details = copy.deepcopy(workflow_spec)
-        if workflow_spec.get('version') not in range(2, 6):
-            raise DXError("Unrecognized workflow version {v} in {w}\n".format(v=workflow_spec.get('version', '<none>'), w=self))
-
-        launched_jobs = OrderedDict()
-        for stage in workflow_spec['stages']:
-            launched_jobs[stage['id']] = None
-
-        for k in range(len(workflow_spec['stages'])):
-            workflow_spec['stages'][k].setdefault('key', str(k))
-            for i in workflow_spec['stages'][k].get('inputs', {}).keys():
-                if workflow_spec['stages'][k]['inputs'][i] == "":
-                    del workflow_spec['stages'][k]['inputs'][i]
-
-        for k, stage in enumerate(workflow_spec['stages']):
-            inputs_from_stage = {k: stage_to_job_refs(v, launched_jobs) for k, v in stage['inputs'].iteritems() if v is not None}
-
-            exec_id = stage['app']['id'] if 'id' in stage['app'] else stage['app']
-            if isinstance(exec_id, dict) and '$dnanexus_link' in exec_id:
-                exec_id = exec_id['$dnanexus_link']
-            if exec_id.startswith('app-'):
-                from dxpy.utils.resolver import get_app_from_path
-                exec_id = get_app_from_path(exec_id)['id']
-
-            executable = get_handler(exec_id)
-            executable_desc = executable.describe()
-
-            if exec_id.startswith('app-'):
-                workflow_details['stages'][k]['app'] = {
-                    "$dnanexus_link": 'app-' + executable_desc['name'] + '/' + executable_desc['version']
-                }
-
-            job_name = executable_desc.get('title', '')
-            if job_name == '':
-                job_name = executable_desc['name']
-            job_name += ' - ' + (name if name is not None else workflow_name)
-
-            exec_inputs = ExecutableInputs(executable, input_name_prefix=str(stage['key'])+".")
-            exec_inputs.update(inputs_from_stage, strip_prefix=False)
-            fake_args = Namespace()
-            fake_args.filename = None
-            fake_args.input = None
-            fake_args.input_spec = None
-            fake_args.input_json = json.dumps(workflow_input)
-            exec_inputs.update_from_args(fake_args)
-            input_json = exec_inputs.inputs
-
-            launched_jobs[stage['id']] = executable.run(input_json, project=project, folder=folder,
-                                                        name=job_name,
-                                                        **kwargs)
-
-        # Update workflow with updated executable IDs
-        self.set_details(workflow_details)
-
-        return launched_jobs.values()
-
-######################
-# DXAnalysisWorkflow #
-######################
+##############
+# DXWorkflow #
+##############
 
 def new_dxworkflow(title=None, summary=None, description=None, output_folder=None, init_from=None, **kwargs):
     '''
@@ -164,8 +44,8 @@ def new_dxworkflow(title=None, summary=None, description=None, output_folder=Non
     :param output_folder: Default output folder of the workflow (optional)
     :type output_folder: string
     :param init_from: Another analysis workflow object handler or and analysis (string or handler) from which to initialize the metadata (optional)
-    :type init_from: :class:`~dxpy.bindings.dxworkflow.DXAnalysisWorkflow`, :class:`~dxpy.bindings.dxanalysis.DXAnalysis`, or string (for analysis IDs only)
-    :rtype: :class:`DXAnalysisWorkflow`
+    :type init_from: :class:`~dxpy.bindings.dxworkflow.DXWorkflow`, :class:`~dxpy.bindings.dxanalysis.DXAnalysis`, or string (for analysis IDs only)
+    :rtype: :class:`DXWorkflow`
 
     Additional optional parameters not listed: all those under
     :func:`dxpy.bindings.DXDataObject.new`, except `details`.
@@ -179,14 +59,14 @@ def new_dxworkflow(title=None, summary=None, description=None, output_folder=Non
 
     Note that this function is shorthand for::
 
-        dxworkflow = DXAnalysisWorkflow()
+        dxworkflow = DXWorkflow()
         dxworkflow.new(**kwargs)
     '''
-    dxworkflow = DXAnalysisWorkflow()
+    dxworkflow = DXWorkflow()
     dxworkflow.new(title=title, summary=summary, description=description, output_folder=output_folder, init_from=init_from, **kwargs)
     return dxworkflow
 
-class DXAnalysisWorkflow(DXDataObject, DXExecutable):
+class DXWorkflow(DXDataObject, DXExecutable):
     '''
     Remote workflow object handler.  This class is used for the
     workflow class data objects which produce an analysis when run.
@@ -220,17 +100,17 @@ class DXAnalysisWorkflow(DXDataObject, DXExecutable):
         :param output_folder: Default output folder of the workflow (optional)
         :type output_folder: string
         :param init_from: Another analysis workflow object handler or and analysis (string or handler) from which to initialize the metadata (optional)
-        :type init_from: :class:`~dxpy.bindings.dxworkflow.DXAnalysisWorkflow`, :class:`~dxpy.bindings.dxanalysis.DXAnalysis`, or string (for analysis IDs only)
+        :type init_from: :class:`~dxpy.bindings.dxworkflow.DXWorkflow`, :class:`~dxpy.bindings.dxanalysis.DXAnalysis`, or string (for analysis IDs only)
 
         Create a new remote workflow object.
         """
 
         if "init_from" in kwargs:
             if kwargs["init_from"] is not None:
-                if not (isinstance(kwargs["init_from"], (DXAnalysisWorkflow, DXAnalysis)) or \
+                if not (isinstance(kwargs["init_from"], (DXWorkflow, DXAnalysis)) or \
                         (isinstance(kwargs["init_from"], basestring) and \
                          re.compile('^analysis-[0-9A-Za-z]{24}$').match(kwargs["init_from"]))):
-                    raise DXError("Expected init_from to be an instance of DXAnalysisWorkflow or DXAnalysis, or to be a string analysis ID.")
+                    raise DXError("Expected init_from to be an instance of DXWorkflow or DXAnalysis, or to be a string analysis ID.")
                 if isinstance(kwargs["init_from"], basestring):
                     dx_hash["initializeFrom"] = {"id": kwargs["init_from"]}
                 else:
@@ -269,27 +149,43 @@ class DXAnalysisWorkflow(DXDataObject, DXExecutable):
 
     def _get_stage_id(self, stage):
         '''
-        :param stage: Either a number (for the nth stage, starting from 0), or a stage ID
+        :param stage: A stage ID, name, or index (stage index is the number n for the nth stage, starting from 0; can be provided as an int or a string)
         :type stage: int or string
-        :returns: The stage ID (this is a no-op if it was already a string)
+        :returns: The stage ID (this is a no-op if it was already a stage ID)
         :raises: :class:`~dxpy.exceptions.DXError` if *stage* could not be parsed or resolved to a stage ID
         '''
-        stage_id = None
+        # first, if it is a string, see if it is an integer
         if isinstance(stage, basestring):
-            stage_id = stage
-        else:
+            try:
+                stage = int(stage)
+            except:
+                # we'll try parsing it as a string later
+                pass
+
+        if not isinstance(stage, basestring):
+            # Try to parse as stage index; ensure that if it's not a
+            # string that it is an integer at this point.
             try:
                 stage_index = int(stage)
             except:
-                raise DXError('DXAnalysisWorkflow: "stage" was neither a string stage ID nor an integer index')
+                raise DXError('DXWorkflow: the given stage identifier was neither a string stage ID nor an integer index')
             if stage_index < 0 or stage_index >= len(self.stages):
-                raise DXError('DXAnalysisWorkflow: the workflow contains ' + str(len(self.stages)) + ' stage(s), and the provided value for "stage" is out of range')
-            stage_id = self.stages[stage_index].get("id")
+                raise DXError('DXWorkflow: the workflow contains ' + str(len(self.stages)) + \
+                              ' stage(s), and the numerical value of the given stage identifier is out of range')
+            return self.stages[stage_index].get("id")
 
-        if re.compile('^stage-[0-9A-Za-z]{24}$').match(stage_id) is None:
-            raise DXError('DXAnalysisWorkflow: "stage" did not resolve to a properly formed stage ID')
-
-        return stage_id
+        if re.compile('^stage-[0-9A-Za-z]{24}$').match(stage) is None:
+            # Doesn't look like a stage ID, so look for it as a name
+            matching_stage_ids = [stg['id'] for stg in self.stages if stg.get('name') == stage]
+            if len(matching_stage_ids) == 0:
+                raise DXError('DXWorkflow: the given stage identifier could not be parsed as a stage ID nor found as a stage name')
+            elif len(matching_stage_ids) > 1:
+                raise DXError('DXWorkflow: more than one workflow stage was found to have the name "' + stage + '"')
+            else:
+                return matching_stage_ids[0]
+        else:
+            # Already a stage ID
+            return stage
 
     def add_stage(self, executable, name=None, folder=None, stage_input=None, instance_type=None,
                   edit_version=None, **kwargs):
@@ -336,7 +232,7 @@ class DXAnalysisWorkflow(DXDataObject, DXExecutable):
 
     def get_stage(self, stage, **kwargs):
         '''
-        :param stage: Either a number (for the nth stage, starting from 0), or a stage ID to describe
+        :param stage: A number for the stage index (for the nth stage, starting from 0), or a string of the stage index, name, or ID
         :type stage: int or string
         :returns: Hash of stage descriptor in workflow
         '''
@@ -348,7 +244,7 @@ class DXAnalysisWorkflow(DXDataObject, DXExecutable):
 
     def remove_stage(self, stage, edit_version=None, **kwargs):
         '''
-        :param stage: Either a number (for the nth stage, starting from 0), or a stage ID to remove
+        :param stage: A number for the stage index (for the nth stage, starting from 0), or a string of the stage index, name, or ID
         :type stage: int or string
         :param edit_version: if provided, the edit version of the workflow that should be modified; if not provided, the current edit version will be used (optional)
         :type edit_version: int
@@ -368,7 +264,7 @@ class DXAnalysisWorkflow(DXDataObject, DXExecutable):
 
     def move_stage(self, stage, new_index, edit_version=None, **kwargs):
         '''
-        :param stage: Either a number (for the nth stage, starting from 0), or a stage ID to remove
+        :param stage: A number for the stage index (for the nth stage, starting from 0), or a string of the stage index, name, or ID
         :type stage: int or string
         :param new_index: The new position in the order of stages that the specified stage should have (where 0 indicates the first stage)
         :type new_index: int
@@ -441,7 +337,7 @@ class DXAnalysisWorkflow(DXDataObject, DXExecutable):
                      name=None, unset_name=False, folder=None, unset_folder=False, stage_input=None,
                      instance_type=None, edit_version=None, **kwargs):
         '''
-        :param stage: Either a number (for the nth stage, starting from 0), or a stage ID to remove
+        :param stage: A number for the stage index (for the nth stage, starting from 0), or a string stage index, name, or ID
         :type stage: int or string
         :param executable: string or a handler for an app or applet
         :type executable: string, DXApplet, or DXApp
@@ -510,35 +406,69 @@ class DXAnalysisWorkflow(DXDataObject, DXExecutable):
                 self.describe() # update cached describe
 
     def _get_input_name(self, input_str):
-        if '.' in input_str and not input_str.startswith('stage-'):
-            stage_index = int(input_str[:input_str.find('.')])
-            return self.stages[stage_index]['id'] + input_str[input_str.find('.'):]
-        else:
-            return input_str
+        '''
+        :param input_str: A string of one of the forms: "<exported input field name>", "<stage ID>.<input field name>", "<stage index>.<input field name>", "<stage name>.<input field name>"
+        :type input_str: string
+        :returns: If the given form was one of those which uses the stage index or stage name, it is translated to the stage ID for use in the API call (stage name takes precedence)
+        '''
+        if '.' in input_str:
+            stage_identifier = input_str[:input_str.find('.')]
+            dot_input_name = input_str[input_str.find('.'):]
+            # Try to parse as a stage ID or name
+            return self._get_stage_id(stage_identifier) + dot_input_name
+
+        return input_str
 
     def _get_effective_input(self, workflow_input):
         effective_input = {}
         for key in workflow_input:
-            effective_input[self._get_input_name(key)] = workflow_input[key]
+            input_name = self._get_input_name(key)
+            if input_name in effective_input:
+                raise DXError('DXWorkflow: the input for ' + input_name + ' was provided more than once')
+            effective_input[input_name] = workflow_input[key]
         return effective_input
 
+    def _get_run_input(self, workflow_input, **kwargs):
+        effective_workflow_input = self._get_effective_input(workflow_input)
+
+        run_input = DXExecutable._get_run_input_common_fields(effective_workflow_input, **kwargs)
+
+        if kwargs.get('stage_instance_types') is not None:
+            run_input['stageSystemRequirements'] = {}
+            for stage, value in kwargs['stage_instance_types'].items():
+                if stage != '*':
+                    stage = self._get_stage_id(stage)
+                run_input['stageSystemRequirements'][stage] = DXExecutable._inst_type_to_sys_reqs(value)
+
+        return run_input
+
+    def _run_impl(self, run_input, **kwargs):
+        return DXAnalysis(dxpy.api.workflow_run(self._dxid, run_input, **kwargs)["id"])
+
     def run(self, workflow_input, *args, **kwargs):
         '''
-        :param workflow_input: Hash of the workflow's input arguments; see below for more details
+        :param workflow_input: Dictionary of the workflow's input arguments; see below for more details
         :type workflow_input: dict
+        :param instance_type: Instance type on which all stages' jobs will be run, or a dict mapping function names to instance types. These may be overridden on a per-stage basis if stage_instance_types is specified.
+        :type instance_type: string or dict
+        :param stage_instance_types: A dict mapping stage IDs, names, or indices to either a string (representing an instance type to be used for all functions in that stage), or a dict mapping function names to instance types.
+        :type stage_instance_types: dict
         :returns: Object handler of the newly created analysis
         :rtype: :class:`~dxpy.bindings.dxanalysis.DXAnalysis`
 
         Run the associated workflow.
 
-        When providing input for the workflow, keys should be of one of the following three forms:
+        When providing input for the workflow, keys should be of one of the following forms:
 
         * "N.name" where *N* is the stage number, and *name* is the
           name of the input, e.g. "0.reads" if the first stage takes
           in an input called "reads"
 
+        * "stagename.name" where *stagename* is the stage name, and
+          *name* is the name of the input within the stage
+
         * "stageID.name" where *stageID* is the stage ID, and *name*
-          is the name of the input
+          is the name of the input within the stage
 
         * "name" where *name* is the name of an input that has been
           exported for the workflow (this name will appear as a key in
@@ -547,24 +477,4 @@ class DXAnalysisWorkflow(DXDataObject, DXExecutable):
 
         See :meth:`dxpy.bindings.dxapplet.DXExecutable.run` for the available args.
         '''
-
-        effective_input = self._get_effective_input(workflow_input)
-        return super(DXAnalysisWorkflow, self).run(effective_input, *args, **kwargs)
-
-##############
-# DXWorkflow #
-##############
-
-def DXWorkflow(dxid, project=None):
-    '''
-    Returns the appropriate remote workflow object handler.
-    '''
-    if dxid is None:
-        # We don't know which subclass to return
-        raise DXError('DXWorkflow requires an ID to return the appropriate handler')
-    if dxid.startswith('record-'):
-        return DXRecordWorkflow(dxid, project)
-    elif dxid.startswith('workflow-'):
-        return DXAnalysisWorkflow(dxid, project)
-    else:
-        raise DXError('DXWorkflow requires a record or workflow ID')
+        return super(DXWorkflow, self).run(workflow_input, *args, **kwargs)
