@@ -19,6 +19,8 @@ Utilities for client-side usage of the streaming log API
 (https://wiki.dnanexus.com/API-Specification-v1.0.0/Logging#API-method%3A-%2Fjob-xxxx%2FgetLog).
 '''
 
+from __future__ import print_function
+
 import json, logging
 
 #from ws4py.client.threadedclient import WebSocketClient
@@ -75,7 +77,7 @@ class DXJobLogStreamClient(WebSocketBaseClient):
                 self.seen_jobs[self.job_id] = {}
             for job_id in self.seen_jobs.keys():
                 self.seen_jobs[job_id] = dxpy.describe(job_id)
-                print get_find_executions_string(self.seen_jobs[job_id], has_children=False, show_outputs=True)
+                print(get_find_executions_string(self.seen_jobs[job_id], has_children=False, show_outputs=True))
         else:
             self.seen_jobs[self.job_id] = dxpy.describe(self.job_id)
 
@@ -87,11 +89,11 @@ class DXJobLogStreamClient(WebSocketBaseClient):
 
         if self.print_job_info and 'job' in message and message['job'] not in self.seen_jobs:
             self.seen_jobs[message['job']] = dxpy.describe(message['job'])
-            print get_find_executions_string(self.seen_jobs[message['job']], has_children=False, show_outputs=False)
+            print(get_find_executions_string(self.seen_jobs[message['job']], has_children=False, show_outputs=False))
 
         if message.get('source') == 'SYSTEM' and message.get('msg') == 'END_LOG':
             self.close()
         elif self.msg_callback:
             self.msg_callback(message)
         else:
-            print self.msg_output_format.format(**message)
+            print(self.msg_output_format.format(**message))
