@@ -876,61 +876,11 @@ class TestDXRecord(unittest.TestCase):
         desc = dxrecord.describe()
         self.assertEqual(desc["folder"], "/a/b/c")
 
-    def test_passhtrough_args(self):
+    def test_passthrough_args(self):
         dxrecord = dxpy.new_dxrecord(auth=dxpy.AUTH_HELPER)
         with self.assertRaises(TypeError):
             dxrecord = dxpy.new_dxrecord(foo=1)
 
-class TestDXWorkflow(unittest.TestCase):
-    def setUp(self):
-        setUpTempProjects(self)
-        self.dxapplet = dxpy.DXApplet()
-        self.dxapplet.new(name="identity-record",
-                          dxapi="1.04",
-                          inputSpec=[{"name": "record", "class": "record"}
-                                     ],
-                          outputSpec=[{"name": "record", "class": "record"}],
-                          runSpec={"code": '''
-@dxpy.entry_point('main')
-def main(record):
-    return {'record': record}''',
-                                   "interpreter": "python2.7"})
-        dxrecord = dxpy.new_dxrecord(name='workflowname',
-                                     details={"stages": [{"job": None,
-                                                          "inputs": {},
-                                                          "app": dxpy.dxlink(self.dxapplet),
-                                                          "id": "stage0-id"
-                                                          },
-                                                         {"job": None,
-                                                          "inputs": {"record": {"connectedTo": {"output": "record",
-                                                                                                "stage": "stage0-id"}
-                                                                                }
-                                                                     },
-                                                          "app": dxpy.dxlink(self.dxapplet),
-                                                          "id": "stage1-id"
-                                                          }],
-                                              "version": 5},
-                                     types=['pipeline'])
-        self.workflow = dxpy.DXWorkflow(dxrecord.get_id())
-        self.closedrecord = dxpy.new_dxrecord(name='a record')
-        self.closedrecord.close()
-
-    def tearDown(self):
-        tearDownTempProjects(self)
-
-    @unittest.skipUnless(testutil.TEST_RUN_JOBS,
-                         'skipping test that would run a job')
-    def test_run_workflow(self):
-        launched_jobs = self.workflow.run({"0.record": dxpy.dxlink(self.closedrecord)})
-        self.assertEqual(len(launched_jobs), 2)
-        job_descs = [dxjob.describe() for dxjob in launched_jobs]
-        self.assertEqual(job_descs[0]['name'], 'identity-record - workflowname')
-        self.assertEqual(get_dxlink_ids(job_descs[0]['input']['record'])[0], self.closedrecord.get_id())
-        self.assertTrue(is_job_ref(job_descs[1]['runInput']['record']))
-        self.assertEqual(get_job_from_jbor(job_descs[1]['runInput']['record']), job_descs[0]['id'])
-        self.assertEqual(get_field_from_jbor(job_descs[1]['runInput']['record']), 'record')
-        launched_jobs[1].wait_on_done()
-
 @unittest.skipUnless(testutil.TEST_RUN_JOBS,
                      'skipping test that would run a job')
 class TestDXAppletJob(unittest.TestCase):
@@ -1008,7 +958,7 @@ def main():
 
         dxjob.terminate()
 
-class TestDXAnalysisWorkflow(unittest.TestCase):
+class TestDXWorkflow(unittest.TestCase):
     def setUp(self):
         setUpTempProjects(self)
 
@@ -1022,7 +972,8 @@ class TestDXAnalysisWorkflow(unittest.TestCase):
         dxapplet = dxpy.DXApplet()
         dxapplet.new(name="test_applet",
                      dxapi="1.04",
-                     inputSpec=[{"name": "number", "class": "int"}],
+                     inputSpec=[{"name": "number", "class": "int"},
+                                {"name": "othernumber", "class": "int"}],
                      outputSpec=[{"name": "number", "class": "int"}],
                      runSpec={"code": '''
 @dxpy.entry_point('main')
@@ -1031,13 +982,17 @@ def main(number):
 ''',
                                "interpreter": "python2.7"})
         stage_id = dxpy.api.workflow_add_stage(dxworkflow.get_id(),
-                                               {"editVersion": 0, "executable": dxapplet.get_id()})['stage']
-        dxanalysis = dxworkflow.run({"0.number": 32})
+                                               {"editVersion": 0,
+                                                "name": "stagename",
+                                                "executable": dxapplet.get_id()})['stage']
+        dxanalysis = dxworkflow.run({"0.number": 32,
+                                     "stagename.othernumber": 42})
         dxanalysis.terminate()
         with self.assertRaises(DXJobFailureError):
             dxanalysis.wait_on_done(timeout=20)
         analysis_desc = dxanalysis.describe()
         self.assertEqual(analysis_desc['input'].get(stage_id + '.number'), 32)
+        self.assertEqual(analysis_desc['input'].get(stage_id + '.othernumber'), 42)
         dxjob = dxpy.DXJob(analysis_desc['stages'][0]['execution']['id'])
         self.assertEqual(dxjob.describe()['input'].get("number"), 32)
 
@@ -1053,9 +1008,78 @@ def main(number):
         self.assertEqual(analysis_desc["tags"], ["foo"])
         self.assertEqual(analysis_desc["properties"], {"foo": "bar"})
 
+    @unittest.skipUnless(testutil.TEST_RUN_JOBS and os.environ.get("DX_RUN_NEXT_TESTS"),
+                         'skipping test that would run a job and requires unreleased features')
+    def test_run_workflow_with_instance_type(self):
+        dxworkflow = dxpy.DXWorkflow(dxpy.api.workflow_new({"project": self.proj_id})['id'])
+        dxapplet = dxpy.DXApplet()
+        dxapplet.new(name="test_applet",
+                     dxapi="1.04",
+                     inputSpec=[],
+                     outputSpec=[],
+                     runSpec={"code": '',
+                              "interpreter": "bash"})
+        stage_id = dxpy.api.workflow_add_stage(dxworkflow.get_id(),
+                                               {"editVersion": 0,
+                                                "name": "stagename",
+                                                "executable": dxapplet.get_id()})['stage']
+        # control (no request)
+        dxanalysis = dxworkflow.run({})
+        time.sleep(2)
+        dxjob = dxpy.DXJob(dxanalysis.describe()['stages'][0]['execution']['id'])
+        self.assertIsNone(dxjob.describe()['instanceType'])
+
+        # request for all stages and all entry points
+        dxanalysis = dxworkflow.run({}, instance_type="dx_m1.medium")
+        time.sleep(2)
+        dxjob = dxpy.DXJob(dxanalysis.describe()['stages'][0]['execution']['id'])
+        self.assertEqual(dxjob.describe()['instanceType'], 'dx_m1.medium')
+
+        # request for all stages, overriding some entry points
+        dxanalysis = dxworkflow.run({}, instance_type={"*": "dx_m1.medium", "foo": "dx_m1.large"})
+        time.sleep(2)
+        dxjob = dxpy.DXJob(dxanalysis.describe()['stages'][0]['execution']['id'])
+        self.assertEqual(dxjob.describe()['instanceType'], 'dx_m1.medium')
+
+        # request for the stage specifically, for all entry points
+        dxanalysis = dxworkflow.run({}, stage_instance_types={stage_id: "dx_m1.large"})
+        time.sleep(2)
+        dxjob = dxpy.DXJob(dxanalysis.describe()['stages'][0]['execution']['id'])
+        self.assertEqual(dxjob.describe()['instanceType'], 'dx_m1.large')
+
+        # request for the stage specifically, overriding some entry points
+        dxanalysis = dxworkflow.run({}, stage_instance_types={stage_id: {"*": "dx_m1.large", "foo": "dx_m1.medium"}})
+        time.sleep(2)
+        dxjob = dxpy.DXJob(dxanalysis.describe()['stages'][0]['execution']['id'])
+        self.assertEqual(dxjob.describe()['instanceType'], 'dx_m1.large')
+
+    @unittest.skipUnless(testutil.TEST_RUN_JOBS, 'skipping test that may run a job')
+    def test_run_workflow_errors(self):
+        dxworkflow = dxpy.DXWorkflow(dxpy.api.workflow_new({"project": self.proj_id})['id'])
+        dxapplet = dxpy.DXApplet()
+        dxapplet.new(name="test_applet",
+                     dxapi="1.04",
+                     inputSpec=[{"name": "number", "class": "int"}],
+                     outputSpec=[{"name": "number", "class": "int"}],
+                     runSpec={"code": '''
+@dxpy.entry_point('main')
+def main(number):
+    raise # Ensure that the applet fails
+''',
+                               "interpreter": "python2.7"})
+        dxworkflow.add_stage(dxapplet, name='stagename')
+
+        # Can't specify the same input more than once (with a
+        # stage-specific syntax)
+        self.assertRaisesRegexp(DXError, 'more than once',
+                                dxworkflow.run, {"0.number": 32, "stagename.number": 42})
+        # Bad stage name
+        self.assertRaisesRegexp(DXError, 'nor found as a stage name',
+                                dxworkflow.run, {"nonexistentstage.number": 32})
+
     def test_new_dxworkflow(self):
         blankworkflow = dxpy.new_dxworkflow()
-        self.assertIsInstance(blankworkflow, dxpy.DXAnalysisWorkflow)
+        self.assertIsInstance(blankworkflow, dxpy.DXWorkflow)
         desc = blankworkflow.describe()
         self.assertEqual(desc['title'], blankworkflow.get_id())
         self.assertEqual(desc['summary'], '')
@@ -1063,7 +1087,7 @@ def main(number):
         self.assertEqual(desc['outputFolder'], None)
 
         dxworkflow = dxpy.new_dxworkflow(title='mytitle', summary='mysummary', description='mydescription', output_folder="/foo")
-        self.assertIsInstance(dxworkflow, dxpy.DXAnalysisWorkflow)
+        self.assertIsInstance(dxworkflow, dxpy.DXWorkflow)
         desc = dxworkflow.describe()
         self.assertEqual(desc['title'], 'mytitle')
         self.assertEqual(desc['summary'], 'mysummary')
@@ -1071,7 +1095,7 @@ def main(number):
         self.assertEqual(desc['outputFolder'], '/foo')
 
         secondworkflow = dxpy.new_dxworkflow(init_from=dxworkflow)
-        self.assertIsInstance(secondworkflow, dxpy.DXAnalysisWorkflow)
+        self.assertIsInstance(secondworkflow, dxpy.DXWorkflow)
         self.assertNotEqual(dxworkflow.get_id(), secondworkflow.get_id())
         desc = secondworkflow.describe()
         self.assertEqual(desc['title'], 'mytitle')
@@ -1096,7 +1120,9 @@ def main(number):
         self.assertEqual(dxworkflow.stages[0]["input"]["my_input"], "hello world")
         self.assertEqual(dxworkflow.stages[0]["systemRequirements"],
                          {"*": {"instanceType": "dx_m1.large"}})
-        second_stage = dxworkflow.add_stage(dxapplet, folder="relativefolder",
+        second_stage = dxworkflow.add_stage(dxapplet,
+                                            name="stagename",
+                                            folder="relativefolder",
                                             instance_type={"main": "dx_m1.large", "foo": "dx_m1.medium"},
                                             edit_version=1)
         self.assertEqual(dxworkflow.editVersion, 2)
@@ -1120,6 +1146,12 @@ def main(number):
         self.assertEqual(dxworkflow.stages[1]["id"], second_stage)
 
         # Remove stages
+
+        # Removing stage by name doesn't work when there's more than
+        # one of that name
+        self.assertRaisesRegexp(DXError, 'more than one workflow stage was found',
+                                dxworkflow.remove_stage, "stagename")
+
         removed_stage = dxworkflow.remove_stage(0)
         self.assertEqual(removed_stage, first_stage)
         self.assertEqual(dxworkflow.editVersion, 5)
@@ -1146,17 +1178,18 @@ def main(number):
         # Add stages
         first_stage = dxworkflow.add_stage(dxapplet, name='stagename', folder="/outputfolder",
                                            stage_input={"my_input": "hello world"})
-        second_stage = dxworkflow.add_stage(dxapplet, name='stagename', folder="/outputfolder",
+        second_stage = dxworkflow.add_stage(dxapplet, name='stagename2', folder="/outputfolder",
                                             stage_input={"my_input": "hello world"})
         # Get stages
-        stage_desc = dxworkflow.get_stage(0)
-        self.assertEqual(stage_desc['id'], first_stage)
-        stage_desc = dxworkflow.get_stage(first_stage)
-        self.assertEqual(stage_desc['id'], first_stage)
-        stage_desc = dxworkflow.get_stage(1)
-        self.assertEqual(stage_desc['id'], second_stage)
-        stage_desc = dxworkflow.get_stage(second_stage)
-        self.assertEqual(stage_desc['id'], second_stage)
+        test_cases = [[0, first_stage],
+                      [first_stage, first_stage],
+                      ['stagename', first_stage],
+                      [1, second_stage],
+                      [second_stage, second_stage],
+                      ['stagename2', second_stage]]
+        for tc in test_cases:
+            stage_desc = dxworkflow.get_stage(tc[0])
+            self.assertEqual(stage_desc['id'], tc[1])
 
         # Errors
         with self.assertRaises(DXError):
@@ -1486,6 +1519,10 @@ class TestDXSearch(unittest.TestCase):
         dxanalysis = dxworkflow.run({stage+".rowFetchChunk": 200},
                                     tags=["foo"],
                                     properties={"foo": "bar"})
+
+        with self.assertRaises(DXError):
+            dxapplet.run(applet_input=prog_input, stage_instance_types={0: "dx_m1.large"})
+
         dxapplet.run(applet_input=prog_input)
         dxjob = dxapplet.run(applet_input=prog_input, tags=["foo", "bar"], properties={"foo": "baz"})
 
